import { Got, OptionsOfJSONResponseBody as GotJSONOptions } from "got";
import { Agent as IAgent, HttpRequest, HttpResponse } from "@ideal-postcodes/core-interface";
interface GotHeaders {
    [key: string]: string | string[] | undefined;
}
/**
 * An optional configuration object which is passed to the underlying got http
 * client
 *
 * `got` is the underlying HTTP client that powers core-node. Be careful when
 * configuring `gotConfig` so as not to manually override critical request
 * attributes like method, query, header, etc. `got` confirmation options
 * available here: [github.com/sindresorhus/got#options](https://github.com/sindresorhus/got#options)
 */
export declare type GotConfig = Partial<GotJSONOptions>;
/**
 * @hidden
 */
interface ToHeader {
    (gotHeaders: GotHeaders): Record<string, string>;
}
/**
 * Converts a Got header object to one that can be used by the client
 *
 * @hidden
 */
export declare const toHeader: ToHeader;
/**
 * Agent
 *
 * Implements node.js HTTP agent for core-interface client
 *
 * @hidden
 */
export declare class Agent implements IAgent {
    got: Got;
    gotConfig: GotConfig;
    constructor(gotConfig?: GotConfig);
    private requestWithBody;
    private request;
    http(httpRequest: HttpRequest): Promise<HttpResponse>;
}
export {};
