"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Agent = exports.toHeader = void 0;
var got_1 = __importDefault(require("got"));
var core_interface_1 = require("@ideal-postcodes/core-interface");
var 
/**
 * Ideal Postcodes base error class implemented in [core-interface](https://core-interface.ideal-postcodes.dev/classes/idealpostcodeserror.html)
 */
IdealPostcodesError = core_interface_1.errors.IdealPostcodesError;
/**
 * Converts a Got header object to one that can be used by the client
 *
 * @hidden
 */
exports.toHeader = function (gotHeaders) {
    return Object.keys(gotHeaders).reduce(function (headers, key) {
        var val = gotHeaders[key];
        if (typeof val === "string") {
            headers[key] = val;
        }
        else if (Array.isArray(val)) {
            headers[key] = val.join(",");
        }
        return headers;
    }, {});
};
/**
 * Adapts got responses to a format consumable by core-interface
 *
 * @hidden
 */
var toHttpResponse = function (httpRequest, response) { return ({
    httpRequest: httpRequest,
    body: response.body,
    httpStatus: response.statusCode || 0,
    header: exports.toHeader(response.headers),
    metadata: { response: response },
}); };
/**
 * Catch non-response errors (e.g. network failure, DNS failure, timeout)
 * wrap in our Error class and return
 *
 * @hidden
 */
var handleError = function (error) {
    var idpcError = new IdealPostcodesError({
        message: "[" + error.name + "] " + error.message,
        httpStatus: 0,
        metadata: { got: error },
    });
    return Promise.reject(idpcError);
};
/**
 * Agent
 *
 * Implements node.js HTTP agent for core-interface client
 *
 * @hidden
 */
var Agent = /** @class */ (function () {
    function Agent(gotConfig) {
        if (gotConfig === void 0) { gotConfig = {}; }
        this.got = got_1.default;
        this.gotConfig = gotConfig;
    }
    Agent.prototype.requestWithBody = function (httpRequest) {
        var body = httpRequest.body, method = httpRequest.method, timeout = httpRequest.timeout, url = httpRequest.url, header = httpRequest.header, query = httpRequest.query;
        return this.got(url, __assign({ method: method, headers: header, searchParams: query, responseType: "json", throwHttpErrors: false, body: body,
            timeout: timeout }, this.gotConfig))
            .then(function (response) { return toHttpResponse(httpRequest, response); })
            .catch(handleError);
    };
    Agent.prototype.request = function (httpRequest) {
        var method = httpRequest.method, timeout = httpRequest.timeout, url = httpRequest.url, header = httpRequest.header, query = httpRequest.query;
        return this.got(url, __assign({ method: method, headers: header, searchParams: query, timeout: timeout, throwHttpErrors: false, responseType: "json" }, this.gotConfig))
            .then(function (response) { return toHttpResponse(httpRequest, response); })
            .catch(handleError);
    };
    Agent.prototype.http = function (httpRequest) {
        if (httpRequest.body !== undefined)
            return this.requestWithBody(httpRequest);
        return this.request(httpRequest);
    };
    return Agent;
}());
exports.Agent = Agent;
